// Example of playing a 24000 Hz 8-bit stereo PCM stream through
// the audio DACs using uDMA.
//
// /Mic, 2012

#include "ngpc.h"
#include "carthdr.h"
#include "library.h"
#include <stdlib.h>
#include <stdio.h>
#include "sample_24000_8_s.h"

typedef volatile u8 vu8;
typedef volatile u16 vu16;

#define Z80_OFF_SOUND_OFF() SOUNDCPU_CTRL = 0xAAAA

#define memcpy(dst, src, len) BlockCopy(dst, src, len)

// Calculate the timer0 count-up threshold based on a T1 prescaler
// and a 24000 Hz sample rate
#define SAMPLE_TIMEOUT (48000/24000)

// The default number of words to transfer before the uDMA
// transfer completion interrupt is triggered
#define DEFAULT_XFER_WORDS	0x10000


void __interrupt udma0int_handler(void);

u32 wordsRemaining = 0, currentTransfer = 0;
u8 *transferSource;


void main(void) {
    InitNGPC();
    SysSetSystemFont();

	Z80_OFF_SOUND_OFF();

	DMA0_INT		= udma0int_handler;

	ClearScreen(SCR_1_PLANE);
	ClearScreen(SCR_2_PLANE);

   	SetColors(SCR_1_PLANE, 0, 0, 0, RGB(0, 0, 0));
   	PrintString(SCR_1_PLANE, 0, 3, 7, "uDMA DAC Test");


	transferSource = (u8*)sample_24000_8_s;

	TRUN	= 0;					// Disable timers
	INTETC01= 6;					// Set prio 6 for the uDMA completion interrupt

	T01MOD 	= TIMER0_SRC_T1 | TIMER01_8BIT;
	TREG0	= SAMPLE_TIMEOUT;
	INTET01	= 0;					// Disable interrupts for timer0; we just want
									// it to trigger uDMA
	TFFCR	= 0;					// Flip-flop (we don't use this)
	TRDC	= 0;					// Double buffering (we don't use this)

	__asm(" DAC_L   equ 0xA2");
	__asm(" DAC_R   equ 0xA3");
	__asm(" WORD_SRC_INC equ 0x09");

	// Set the source, destination, mode and transfer count for uDMA channel 0
	// (a count of 0 means 65536)
	// We set the up the uDMA to transfer 2 bytes each time it's triggered,
	__asm(" ldl xwa, (_transferSource)");
	__asm(" ldc DMAS0,xwa");
	__asm(" ldl xwa,  DAC_L");
	__asm(" ldc DMAD0,xwa");
	__asm(" ldb w,    WORD_SRC_INC");
	__asm(" ldc DMAM0,w");
	__asm(" ldw wa,   0");
	__asm(" ldc DMAC0,wa");

	wordsRemaining 	= sizeof(sample_24000_8_s) / 2;
	currentTransfer = DEFAULT_XFER_WORDS;

	DMA0V	= DMAV_TIMER0;			// Set the uDMA vector for channel 0 to timer0

	// Start timer0
	TRUN	= TIMER0_ON | PRESCALER_ON;

	while (1) {}
}


void __interrupt udma0int_handler(void) {
	// Update the source address and number of bytes remaining, then
	// restart the uDMA
	wordsRemaining -= currentTransfer;
	if (wordsRemaining == 0) {
		// Reached the end of the sample => loop
		transferSource		= (u8*)sample_24000_8_s;
		wordsRemaining 		= sizeof(sample_24000_8_s) / 2;
		currentTransfer 	= DEFAULT_XFER_WORDS;
	} else {
		transferSource 		+= currentTransfer << 1;
		currentTransfer 	= (wordsRemaining >= DEFAULT_XFER_WORDS) ? DEFAULT_XFER_WORDS : wordsRemaining;
	}

	__asm(" ldl xwa,(_transferSource)");
	__asm(" ldc DMAS0,xwa");
	__asm(" ldw wa,(_currentTransfer)");
	__asm(" ldc DMAC0,wa");
	DMA0V	= DMAV_TIMER0;
}

